package com.fonserbc.canvasgame;

import android.os.Bundle;
import android.app.Activity;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.Log;
import android.view.Menu;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.Window;
import android.view.WindowManager;

public class GameActivity extends Activity implements Runnable, SurfaceHolder.Callback {

	/**
	 * Using CANVAS as tag on all verbose logs
	 */
	
	/**
	 * Simple Timer
	 */
	public class Timer {
		public float maxStep = 0.05f;
		private long wallLastTimeStamp = 0;
		private long lastTickMs = 0;
		
		public Timer() {}
		
		/**
		 * When called returns the number of seconds since last call
		 * @return Number of seconds since last call
		 */
		public float tick() {
			long wallCurrent = System.currentTimeMillis();
			lastTickMs = (wallCurrent - wallLastTimeStamp);
			float wallDelta = lastTickMs/1000f;
			wallLastTimeStamp = wallCurrent;
			
			float gameDelta = Math.min(wallDelta, maxStep);
			return gameDelta;
		}
	}
	
	/**
	 * Example Game Object class
	 * 
	 * A changing color square in the middle of the screen
	 */
	public class GameObject {
		
		private static final float SPEED = 100f;
		
		private GameActivity game;
		
		float posX, posY;
		float sizeX, sizeY;
		int dirX, dirY;
		
		Paint paint;
		
		int r, g, b;
		
		public GameObject (GameActivity game) {
			this.game = game;
			
			posX = posY = 0;
			sizeX = sizeY = 0;
			dirX = dirY = 1;
			
			r = g = b = 255;
			paint = new Paint();
			paint.setColor(Color.rgb(r, g, b));
		}
		
		/**
		 * Used to initialize variables that mostly depend on variable values like the screen size, etc
		 */
		public void doStart() {
			sizeX = game.mWidth/4;
			sizeY = game.mHeight/4;
			
			posX = game.mWidth/2 - sizeX/2;
			posY = game.mHeight/2 - sizeY/2;
		}
		
		/**
		 * Used to update the GameObject state
		 * @param deltaTime time since last update
		 */
		public void update (float deltaTime) {
			posX += dirX*SPEED*deltaTime;
			if (posX < 0) {
				posX = 0;
				dirX = 1;
				changeColor();
			}
			else if ((posX + sizeX) > game.mWidth ) {
				posX = game.mWidth - sizeX;
				dirX = -1;
				changeColor();
			}
			
			posY += dirY*SPEED*deltaTime;
			if (posY < 0) {
				posY = 0;
				dirY = 1;
				changeColor();
			}
			else if ((posY + sizeY) > game.mHeight ) {
				posY = game.mHeight - sizeY;
				dirY = -1;
				changeColor();
			}
		}
		
		/**
		 * Used to draw the object in the Canvas
		 * @param c
		 */
		public void doDraw(Canvas c) {
			c.drawRect(posX, posY, posX+sizeX, posY+sizeY, paint);
		}
		
		public void changeColor() {
			r = (int)(Math.random()*256);
			g = (int)(Math.random()*256);
			b = (int)(Math.random()*256);			
			
			paint.setColor(Color.rgb(r, g, b));
		}
	}
	
	SurfaceView gameView;
		SurfaceHolder mSurfaceHolder;
		int mWidth;
		int mHeight;
		boolean firstStart = true;
		boolean surfaceDestroyed = true;

	Thread thread;
		volatile boolean mAlive = true;
		volatile boolean mRun = false;
		
	Timer timer;
	
	GameObject gameObject;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.v("Canvas", "Activity onCreate");
        
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        
        setContentView(R.layout.game_layout);
        
        init();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.game_layout, menu);
        return true;
    }

    protected void onResume() {
    	Log.v("CANVAS", "onResume");
    	super.onResume();
    	
    	gameView = (SurfaceView) findViewById(R.id.game_layout);
    	mSurfaceHolder = gameView.getHolder();
    	mSurfaceHolder.setSizeFromLayout();
    	mSurfaceHolder.addCallback(this);
    	gameView.setFocusable(true);
    	gameView.requestFocus();
    	
    	stopThread();
    	thread = new Thread(this);
    	mAlive = true;
    	if (!surfaceDestroyed) mRun = true;
    	thread.start();
    }
    
    protected void onPause() {
    	Log.v("CANVAS", "onPause");
    	stopThread();
    	super.onPause();
    }
    
    private void stopThread() {
    	Log.v("CANVAS", "stopThread");
    	if (thread != null) {
			mRun = false;
			mAlive = false;

			boolean retry = true;
	        while (retry) {
	            try {
	                thread.join();
	                retry = false;
	            } catch (InterruptedException e) {
	            }
	        }
	        thread = null;
    	}
	}
    
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
		surfaceDestroyed = false;
		mWidth = width;
		mHeight = height;
		mRun = true;
		
		if (firstStart) {
			doStart();
			firstStart = false;
		}
	}

	public void surfaceCreated(SurfaceHolder holder) {
		gameView.requestFocus();
	}

	public void surfaceDestroyed(SurfaceHolder holder) {
		surfaceDestroyed = true;
		mRun = false;
	}
	
	/**
	 * Called when the activity is created. Creates every non surface dependent objects.
	 */
	public void init() {
		Log.v("CANVAS", "Init");
		
		timer = new Timer();
		
		gameObject = new GameObject(this);
	}
	
	/**
	 * Used to initialize variables that mostly depend on surface values like the screen size, etc
	 */
	public void doStart() {
		gameObject.doStart();
	}

	/**
	 * Called when "Thread.start()"
	 */
	public void run() {
		while (mAlive) {
			if (!mRun) try { Thread.sleep(100); } catch (InterruptedException ie) {}

	        while (mRun) {
	            Canvas c = null;
	            try {
	            	synchronized (mSurfaceHolder) {
	            		c = mSurfaceHolder.lockCanvas();
	            		
	            		update();
	                    doDraw(c);
	                }
	            } finally {
	                if (c != null) {
	                    mSurfaceHolder.unlockCanvasAndPost(c);
	                }
	            }
	        }
		}
	}
	
	/**
	 * Called once per tick to update all the Game Objects
	 */
	public void update() {
		float deltaTime = timer.tick();
		
		gameObject.update(deltaTime);
	}
	
	/**
	 * Called once per tick to draw all the Game Objects on the canvas
	 * @param c
	 */
	public void doDraw(Canvas c) {
		
		c.drawColor(Color.BLACK); // Redraw background
		
		gameObject.doDraw(c);
	}
}
